package cn.com.qmth.examcloud.web.actuator;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.actuate.endpoint.annotation.Endpoint;
import org.springframework.boot.actuate.endpoint.annotation.ReadOperation;
import org.springframework.boot.actuate.endpoint.annotation.Selector;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.MediaType;

import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.util.FreeMarkerUtil;
import cn.com.qmth.examcloud.commons.util.ResourceLoader;

/**
 * 接口状态
 *
 * @author WANGWEI
 * @date 2019年7月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Configuration
@Endpoint(id = "api-status")
public class ApiStatusEndpoint {

	@Autowired
	ApiStatusInfoCollector apiStatusInfoCollector;

	/**
	 * url: /actuator/api-status
	 *
	 * @author WANGWEI
	 * @return
	 */
	@ReadOperation(produces = {MediaType.TEXT_HTML_VALUE})
	public String getByDefault() {
		return get("");
	}

	/**
	 * url: /actuator/api-status/ignored?order=XX
	 *
	 * @author WANGWEI
	 * @param order
	 * @return
	 */
	@ReadOperation(produces = {MediaType.TEXT_HTML_VALUE})
	public String get(@Selector String order) {

		List<ApiStatusInfo> list = ApiStatusInfoHolder.getApiStatusInfoList();

		Collections.sort(list, new Comparator<ApiStatusInfo>() {
			@Override
			public int compare(ApiStatusInfo o1, ApiStatusInfo o2) {

				try {
					Field field = o1.getClass().getDeclaredField(order);
					field.setAccessible(true);
					Object value1 = field.get(o1);
					Object value2 = field.get(o2);

					if (null == value1) {
						return -1;
					} else if (null == value2) {
						return 1;
					} else if (value1 instanceof Long) {
						return (int) (((Long) value2) - ((Long) value1));
					} else if (value1 instanceof Double) {
						return (int) (((Double) value2) - ((Double) value1));
					} else if (value1 instanceof String) {
						return ((String) value1).compareToIgnoreCase((String) value2);
					} else {
						return 0;
					}
				} catch (Exception e) {
					return 0;
				}
			}
		});

		Map<String, Object> map = Maps.newHashMap();
		map.put("data", list);
		map.put("dateTime", new Date());

		String html = ResourceLoader.getResource("api-status.html");

		String ret = FreeMarkerUtil.process(html, map);
		return ret;
	}

}
