package cn.com.qmth.examcloud.web.actuator;

import java.util.List;
import java.util.Map;

import org.springframework.stereotype.Component;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.util.Calculator;
import cn.com.qmth.examcloud.web.support.ApiInfo;
import cn.com.qmth.examcloud.web.support.ApiInfoHolder;

@Component
public class ApiStatusInfoCollector {

	public synchronized List<ApiStatusInfo> collect(ReportInfo reportInfo) {

		List<HistogramInfo> histogramInfoList = reportInfo.getHistogramInfoList();
		List<MeterInfo> meterInfoList = reportInfo.getMeterInfoList();

		Map<String, MeterInfo> apiErrMeterInfoMap = Maps.newHashMap();
		for (MeterInfo meterInfo : meterInfoList) {
			String key = meterInfo.getKey();
			if (key.startsWith(MetricNames.API_ERROR_METER.name())) {
				String mapping = key.substring(MetricNames.API_ERROR_METER.name().length() + 1);
				apiErrMeterInfoMap.put(mapping, meterInfo);
			}
		}

		Map<String, HistogramInfo> apiHistogramInfoMap = Maps.newHashMap();
		for (HistogramInfo histogramInfo : histogramInfoList) {
			String key = histogramInfo.getKey();
			if (key.startsWith(MetricNames.API_HISTOGRAM.name())) {
				String mapping = key.substring(MetricNames.API_HISTOGRAM.name().length() + 1);
				apiHistogramInfoMap.put(mapping, histogramInfo);
			}
		}

		List<ApiStatusInfo> list = Lists.newArrayList();

		for (MeterInfo cur : meterInfoList) {
			String key = cur.getKey();
			if (key.startsWith(MetricNames.API_METER.name())) {

				String mapping = key.substring(MetricNames.API_METER.name().length() + 1);

				ApiStatusInfo apiStatusInfo = new ApiStatusInfo();
				list.add(apiStatusInfo);

				apiStatusInfo.setMapping(mapping);

				ApiInfo apiInfo = ApiInfoHolder.getApiInfo(mapping);
				if (null != apiInfo) {
					apiStatusInfo.setDescription(apiInfo.getDescription());
				}

				apiStatusInfo.setCount(cur.getCount());
				apiStatusInfo.setMeanRate(cur.getMeanRate());
				apiStatusInfo.setOneMinuteRate(cur.getOneMinuteRate());
				apiStatusInfo.setFiveMinuteRate(cur.getFiveMinuteRate());
				apiStatusInfo.setFifteenMinuteRate(cur.getFifteenMinuteRate());

				HistogramInfo histogramInfo = apiHistogramInfoMap.get(mapping);

				apiStatusInfo.setMin(histogramInfo.getMin());
				apiStatusInfo.setMax(histogramInfo.getMax());
				apiStatusInfo.setMean(histogramInfo.getMean());

				apiStatusInfo.setP50(histogramInfo.getP50());
				apiStatusInfo.setP75(histogramInfo.getP75());
				apiStatusInfo.setP95(histogramInfo.getP95());
				apiStatusInfo.setP98(histogramInfo.getP98());
				apiStatusInfo.setP99(histogramInfo.getP99());

				MeterInfo apiErrMeterInfo = apiErrMeterInfoMap.get(mapping);
				if (null == apiErrMeterInfo) {
					apiStatusInfo.setErrorCount(0L);
					apiStatusInfo.setErrorMeanRate(0L);
					apiStatusInfo.setErrorPercent(0D);
				} else {
					apiStatusInfo.setErrorCount(apiErrMeterInfo.getCount());
					apiStatusInfo.setErrorMeanRate(apiErrMeterInfo.getMeanRate());
					apiStatusInfo.setErrorPercent(
							getPercent(apiErrMeterInfo.getCount(), cur.getCount()));
				}

			}
		}

		return list;
	}

	private Double getPercent(double v1, double v2) {
		if (0 == v1 || 0 == v2) {
			return 0d;
		}
		double ret = Calculator.divide(v1, v2, 3);
		return ret;
	}

}
