package cn.com.qmth.examcloud.web.actuator;

import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

import org.apache.commons.lang3.RandomUtils;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

@Component
@Order(1000)
public class ApiStatusInfoHolder implements ApplicationRunner {

	private static final ExamCloudLog LOG = ExamCloudLogFactory.getLog(ApiStatusInfoHolder.class);

	private static Map<String, ApiStatusInfo> apiStatusInfoMap = Maps.newHashMap();

	private static List<ApiStatusInfo> apiStatusInfoList = Lists.newArrayList();

	private static AtomicBoolean running = new AtomicBoolean(false);

	public static ApiStatusInfo getApiStatusInfo(String mapping) {
		return apiStatusInfoMap.get(mapping);
	}

	public static List<ApiStatusInfo> getApiStatusInfoList() {
		if (running.get()) {
			return apiStatusInfoList;
		}
		refresh();
		return apiStatusInfoList;
	}

	@Override
	public void run(ApplicationArguments args) throws Exception {

		new Thread(new Runnable() {
			@Override
			public void run() {
				while (true) {
					Util.sleep(5);
					refresh();
				}
			}

		}).start();
	}

	private static synchronized void refresh() {
		long trace = 10000 + RandomUtils.nextLong(0, 89999);
		LOG.debug(trace + " [ApiStatus]. refresh...");
		running.set(true);
		try {
			ReportorHolder.getApiDataReportor().report();

			Util.sleep(TimeUnit.MILLISECONDS, 500);

			ReportInfo reportInfo = ReportorHolder.getApiDataReportor().getReportInfo();

			ApiStatusInfoCollector apiStatusInfoCollector = SpringContextHolder
					.getBean(ApiStatusInfoCollector.class);
			apiStatusInfoList = apiStatusInfoCollector.collect(reportInfo);

			Map<String, ApiStatusInfo> newApiInfoMap = apiStatusInfoList.stream()
					.collect(Collectors.toMap(ApiStatusInfo::getMapping, info -> info));

			apiStatusInfoMap = newApiInfoMap;

		} catch (Exception e) {
			LOG.debug(trace + " [ApiStatus]. fail to refresh API status", e);
		}

		running.set(false);
		LOG.debug(trace + " [ApiStatus]. OVER");
	}

}
