package cn.com.qmth.examcloud.web.cloud;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import com.netflix.loadbalancer.IPing;
import com.netflix.loadbalancer.IRule;
import com.netflix.loadbalancer.PingUrl;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.web.bootstrap.PropertyHolder;

/**
 * RibbonClients 配置
 *
 * @author WANGWEI
 * @date 2019年1月28日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
@Configuration
public class RibbonClientsConfiguration {

	private static final ExamCloudLog DEBUG_LOG = ExamCloudLogFactory
			.getLog(CloudClientConfiguration.class);

	@Bean
	public IRule ribbonRule() {

		String rule = PropertyHolder.getString("examcloud.rpc.loadbalance.rule", "BestAvailableRule");

		if (rule.equals("RoundRobinRule")) {
			return new com.netflix.loadbalancer.RoundRobinRule();
		} else if (rule.equals("ZoneAvoidanceRule")) {
			return new com.netflix.loadbalancer.ZoneAvoidanceRule();
		} else if (rule.equals("RetryRule")) {
			return new com.netflix.loadbalancer.RetryRule();
		} else if (rule.equals("RandomRule")) {
			return new com.netflix.loadbalancer.RandomRule();
		} else if (rule.equals("BestAvailableRule")) {
			return new com.netflix.loadbalancer.BestAvailableRule();
		} else if (rule.equals("WeightedResponseTimeRule")) {
			return new com.netflix.loadbalancer.WeightedResponseTimeRule();
		} else {
			DEBUG_LOG.info(
					"value of property[examcloud.rpc.balance.rule] is wrong. will use default value [BestAvailableRule] .");
			return new com.netflix.loadbalancer.WeightedResponseTimeRule();
		}
	}

	@Bean
	public IPing ribbonPing() {
		return new PingUrl();
	}

}
