package cn.com.qmth.examcloud.web.druid;

import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;

import com.alibaba.druid.pool.DruidDataSource;

import cn.com.qmth.examcloud.web.druid.properties.DruidStatProperties;
import cn.com.qmth.examcloud.web.druid.stat.DruidFilterConfiguration;
import cn.com.qmth.examcloud.web.druid.stat.DruidSpringAopConfiguration;
import cn.com.qmth.examcloud.web.druid.stat.DruidStatViewServletConfiguration;
import cn.com.qmth.examcloud.web.druid.stat.DruidWebStatFilterConfiguration;

@Configuration
@ConditionalOnClass(DruidDataSource.class)
@AutoConfigureBefore(DataSourceAutoConfiguration.class)
@EnableConfigurationProperties({DruidStatProperties.class, DataSourceProperties.class})
@Import({DruidSpringAopConfiguration.class, DruidStatViewServletConfiguration.class,
		DruidWebStatFilterConfiguration.class, DruidFilterConfiguration.class})
public class DruidDataSourceAutoConfigure {

	private static final Logger LOGGER = LoggerFactory
			.getLogger(DruidDataSourceAutoConfigure.class);

	@Bean(initMethod = "init")
	@ConditionalOnMissingBean
	@ConditionalOnProperty(name = "spring.datasource.type", havingValue = "com.alibaba.druid.pool.DruidDataSource", matchIfMissing = false)
	public DataSource dataSource() {
		LOGGER.info("Init DruidDataSource");
		return new DruidDataSourceWrapper();
	}
}
