package cn.com.qmth.examcloud.web.helpers.tree;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.exception.ExamCloudRuntimeException;
import cn.com.qmth.examcloud.commons.util.JsonUtil;

/**
 * 树工具
 *
 * @author WANG WEI
 */
public class TreeUtil {

	/**
	 * 转换
	 * 
	 * @author WANG WEI
	 * @param treeNode
	 * @param c
	 * @return
	 */
	public static TreeNode convert(TreeNode treeNode, Class<? extends TreeNode> c) {
		try {
			TreeNode instance = c.newInstance();
			instance.setNodeId(treeNode.getNodeId());
			instance.setNodeName(treeNode.getNodeName());
			instance.setParentNodeId(treeNode.getParentNodeId());

			return instance;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * 转换
	 *
	 * @author WANGWEI
	 * @param treeNodeList
	 * @param c
	 * @return
	 */
	public static List<TreeNode> convert(List<? extends TreeNode> treeNodeList,
			Class<? extends TreeNode> c) {
		List<TreeNode> ret = new ArrayList<TreeNode>();
		for (TreeNode n : treeNodeList) {
			ret.add(convert(n, c));
		}
		return ret;
	}

	/**
	 * 转换
	 *
	 * @author WANGWEI
	 * @param root
	 * @param treeNodeList
	 * @param disabledCodeList
	 * @param includeDisabledCodes
	 * @return
	 */
	public static EleTreeNode convert2OneEleTreeNode(EleTreeNode root,
			List<? extends TreeNode> treeNodeList, List<String> disabledCodeList,
			boolean includeDisabledCodes) {

		Map<String, EleTreeNode> eleTreeMap = Maps.newHashMap();
		for (TreeNode cur : treeNodeList) {
			Long parentId = null;
			if (null != cur.getParentNodeId()) {
				parentId = Long.parseLong(cur.getParentNodeId());
			}
			EleTreeNode eleTree = new EleTreeNode(Long.parseLong(cur.getNodeId()),
					cur.getNodeName(), parentId);
			eleTree.setDisabled(disabledCodeList.contains(cur.getNodeCode()));
			eleTreeMap.put(String.valueOf(eleTree.getId()), eleTree);
		}

		List<EleTreeNode> rootChildren = root.getChildren();
		if (null == rootChildren) {
			rootChildren = Lists.newArrayList();
			root.setChildren(rootChildren);
		}

		for (TreeNode cur : treeNodeList) {
			EleTreeNode eleTree = eleTreeMap.get(cur.getNodeId());
			if (null == eleTree || eleTree.getDisabled()) {
				continue;
			}
			String parentTreeId = cur.getParentNodeId();

			if (StringUtils.isBlank(parentTreeId)) {
				rootChildren.add(eleTree);
			} else {
				EleTreeNode parentEleTree = eleTreeMap.get(parentTreeId);
				if (null == parentEleTree) {
					throw new ExamCloudRuntimeException(
							"no parent. TreeNode:" + JsonUtil.toJson(cur));
				}
				if (parentEleTree.getDisabled()) {
					continue;
				}
				List<EleTreeNode> children = parentEleTree.getChildren();
				if (null == children) {
					children = Lists.newArrayList();
					parentEleTree.setChildren(children);
				}
				children.add(eleTree);
			}

		}
		return root;
	}

}
