package cn.com.qmth.examcloud.web.interceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.web.servlet.HandlerInterceptor;

import com.codahale.metrics.Histogram;
import com.codahale.metrics.Meter;
import com.codahale.metrics.MetricRegistry;

import cn.com.qmth.examcloud.web.actuator.MetricNames;
import cn.com.qmth.examcloud.web.actuator.MetricRegistryHolder;
import cn.com.qmth.examcloud.web.enums.HttpServletRequestAttribute;
import cn.com.qmth.examcloud.web.support.ApiInfo;

/**
 * API统计拦截器
 *
 * @author WANGWEI
 * @date 2019年7月24日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class ApiStatisticInterceptor implements HandlerInterceptor {

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
			Object handler) throws Exception {

		request.setAttribute(
				HttpServletRequestAttribute.API_STATISTIC_INTERCEPTOR_START_TIME.name(),
				System.currentTimeMillis());

		ApiInfo apiInfo = (ApiInfo) request
				.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

		if (null == apiInfo) {
			return true;
		}

		return true;
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
			Object handler, Exception ex) throws Exception {

		ApiInfo apiInfo = (ApiInfo) request
				.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

		if (null == apiInfo) {
			return;
		}

		Histogram histogram = MetricRegistryHolder.getDefalut().histogram(
				MetricRegistry.name(MetricNames.API_HISTOGRAM.name(), apiInfo.getMapping()));

		Long startTime = (Long) request.getAttribute(
				HttpServletRequestAttribute.API_STATISTIC_INTERCEPTOR_START_TIME.name());

		histogram.update(System.currentTimeMillis() - startTime);

		Boolean hasException = (Boolean) request
				.getAttribute(HttpServletRequestAttribute.$_EXCEPTION_HAPPENED.name());

		if (null != hasException && hasException) {
			Meter apiErrorMeter = MetricRegistryHolder.getDefalut().meter(
					MetricRegistry.name(MetricNames.API_ERROR_METER.name(), apiInfo.getMapping()));
			apiErrorMeter.mark();
		}

		Meter apiMeter = MetricRegistryHolder.getDefalut()
				.meter(MetricRegistry.name(MetricNames.API_METER.name(), apiInfo.getMapping()));
		apiMeter.mark();

	}
}
