package cn.com.qmth.examcloud.web.task;

import org.apache.logging.log4j.ThreadContext;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.ThreadLocalUtil;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

/**
 * 并行任务分发器
 *
 * @author WANGWEI
 * @date 2018年3月7日
 * @Copyright (c) 2018-2020 WANGWEI [QQ:522080330] All Rights Reserved.
 */
public class DistributionJob implements Job {
	private static final ExamCloudLog TASK_LOG = ExamCloudLogFactory.getLog("TASK_LOGGER");

	@Override
	public void execute(JobExecutionContext context) throws JobExecutionException {
		String traceId = ThreadLocalUtil.next();
		ThreadContext.put("TRACE_ID", traceId);
		ScheduleJob scheduleJob = null;
		try {
			scheduleJob = (ScheduleJob) context.getMergedJobDataMap().get("scheduleJob");

			if (TASK_LOG.isDebugEnabled()) {
				TASK_LOG.debug("distribute job. job detail :" + JsonUtil.toJson(scheduleJob));
			}
			Object bean = SpringContextHolder.getBean(scheduleJob.getSpringBean());

			Task task = (Task) bean;
			task.execute(scheduleJob, traceId);
		} catch (Exception e) {
			if (TASK_LOG.isErrorEnabled()) {
				TASK_LOG.error(
						"fail to distribute job. job detail :" + JsonUtil.toJson(scheduleJob), e);
			}
			throw new JobExecutionException(e);
		} finally {
			// 清理log4j线程上下文
			ThreadContext.clearAll();
		}

	}

}
